#include "knots2.h"
#include<iostream>
#include"point.h"
#include"vector.h"
#include<iomanip>
#include<ctype.h>
#include"vectors.h"
#include"shortknot.h"
#include<fstream>

using namespace std;

knots::knots():numSides(1), vertex(1), lenvtr(1), side(1), angle(1)
{
  minRad_factor = 1;
}

knots::knots(int numberOfSides):
  numSides(numberOfSides), vertex(numberOfSides), 
  lenvtr(numberOfSides), side(numberOfSides), angle(numberOfSides)
{
  minRad_factor = 1;
}

// knots::knots(int p, int q, double bigRadius,
// 	     double littleRadius, int numberOfVertices): 
//              numSides(numberOfVertices), vertex(numberOfVertices)
// {
//   double h=p+q+bigRadius+littleRadius+numberOfVertices;
// }

knots::knots(const knots & source): numSides(source.vnum()), 
  vertex(source.vnum()), lenvtr(source.vnum()), side(source.vnum()),
  angle(source.vnum())
{
  int i;

  for (i=0; i<numSides; i++)
    {
      vertex[i] = source[i];
      lenvtr[i] = source.lenvtr[i];
      side[i] = source.side[i];
      angle[i] = source.angle[i];
    }

  for (i=0; i<4; i++)
    R[i] = source.R[i];

  injRadius = source.injRadius;
  epsilon = source.epsilon;
  polyLength = source.polyLength;
  minRad_factor = source.minRadFactor();
}

knots & knots::operator =(const knots & source)
{
  if (this != &source)
    {
      if(numSides != source.vnum())
	{
	  numSides = source.vnum();
	  vertex.setSize(numSides);
	  lenvtr.SetSize(numSides);
	  side.setSize(numSides);
	  angle.SetSize(numSides);
	}

      for (int i=0; i<numSides; i++)
	{
	  vertex[i] = source[i];
	  lenvtr[i] = source.lenvtr[i];
	  side[i] = source.side[i];
	  angle[i] = source.angle[i];
	}  

      for (int i=0; i<4; i++)
	R[i] = source.R[i];
      
      injRadius = source.injRadius;
      epsilon = source.epsilon;
      polyLength = source.polyLength;
      minRad_factor = source.minRad_factor;
    }
  
  return *this;
}

knots::~knots()
{
  numSides = 0;
  vertex.setSize(0);
  lenvtr.SetSize(0);
  side.setSize(0);
  angle.SetSize(0);
}

const point & knots::operator [] (const int & index) const
{
  return vertex[index];
}

point & knots::operator [](const int & index)
{
  return vertex[index];
}

int knots::setSize(int numberOfSides)
{
  numSides = numberOfSides;
  lenvtr.SetSize(numberOfSides);
  side.setSize(numberOfSides);
  angle.SetSize(numberOfSides);
  return vertex.setSize(numberOfSides);
}

int knots::vnum() const
{
  return numSides;
}

ostream & operator << (ostream & out, const knots & source)
{
  int top = source.vnum();

  out << "{" << top << "}\n";

  out.setf(ios::fixed);
  out.precision(19);

  for (int i=0; i<top; i++)
    out << source[i][0] << ",\t" << source[i][1] << ",\t" 
      << source[i][2] << endl;

  return out;
}

istream & operator >> (istream & in, knots & source)
{
  char comma;
  int vnum;

  /* first extract the number of vertices */

  while( ! isdigit(in.peek()) )
    in.get(comma);

  if(! (in >> vnum))
    {
      cerr << "HELP!\n";
      return in;
    }

  while( (! isdigit(in.peek())) && (in.peek() != '-') )
    in.get(comma);

  source.numSides = vnum;
  source.setSize(vnum);
  source.lenvtr.SetSize(vnum);
  source.side.setSize(vnum);
  source.angle.SetSize(vnum);

  for (int i=0; i<vnum; i++)
    {
      while( (! isdigit(in.peek())) && (in.peek() != '-') &&(in.peek() != '.'))
	in.get(comma);

      if(! (in >> source[i][0]))
	return in;

      while( (! isdigit(in.peek())) && (in.peek() != '-') &&(in.peek() != '.'))
	in.get(comma);

      if(! (in >> source[i][1]))
	return in;

      while( (! isdigit(in.peek())) && (in.peek() != '-') &&(in.peek() != '.'))
	in.get(comma);

      if(! (in >> source[i][2]))
	return in;
    }

  source.Compute();

  return in;
}

int operator ==(const knots & a, const knots & b)
{
  if(a.vnum() != b.vnum())
    return 0;

  int top = a.vnum();

  for (int i=0; i<top; i++)
    for (int j=0; j<3; j++)
      {
	if(a[i][j] != b[i][j])
	  return 0;
      }
  return 1;
}

int operator !=(const knots & a, const knots & b)
{
  return !(a == b);
}

knots operator + (const knots & knot1, const knots & knot2)
{
  if (knot1.vnum() != knot2.vnum() )
    {
      cerr << "Different number of vertices!\n";
      return knot1;
    }

  int top = knot1.vnum();
  knots sum(top);

  for(int i=0; i<top; i++)
    {
      sum[i][0] = knot1[i][0] + knot2[i][0];
      sum[i][1] = knot1[i][1] + knot2[i][1];
      sum[i][2] = knot1[i][2] + knot2[i][2];
    }

  sum.Compute();

  return sum;
}

knots operator * (double num, const knots & source)
{
  int top = source.vnum();
  knots sum(top);

  for(int i=0; i<top; i++)
    {
      sum[i][0] = num * source[i][0];
      sum[i][1] = num * source[i][1];
      sum[i][2] = num * source[i][2];
    }

  sum.Compute();

  return sum;
}  

void knots::rotatexy(const double & theta)
{
  shortknot temp(*this);
  point rotator[3];
  rotator[0][0] = cos(theta);
  rotator[1][1] = rotator[0][0];
  rotator[1][0] = sin(theta);
  rotator[0][1] = -rotator[1][0];
  rotator[0][2] = rotator[1][2] = rotator[2][0] = rotator[2][1] = 0;
  rotator[2][2] = 1;

  int top = numSides;

  for(int i=0; i< top; i++)
    for(int j=0; j<3; j++)
      vertex[i][j] = rotator[j]*temp[i];
}

void knots::rotateyz(const double & angle)
{
  shortknot temp(*this);
  point rotator[3];
  rotator[1][1] = cos(angle);
  rotator[2][2] = rotator[1][1];
  rotator[1][2] = sin(angle);
  rotator[2][1] = -rotator[1][2];
  rotator[1][0] = rotator[2][0] = rotator[0][1] = rotator[0][2] = 0;
  rotator[0][0] = 1;

  int top = numSides;

  for(int i=0; i< top; i++)
    for(int j=0; j<3; j++)
      vertex[i][j] = rotator[j]*temp[i];
}

void knots::rotaterawyz(const double & cosangle)
{
  shortknot temp(*this);
  point rotator[3];
  rotator[1][1] = cosangle;
  rotator[2][2] = rotator[1][1];
  rotator[1][2] = sqrt(1-cosangle*cosangle);
  rotator[2][1] = -rotator[1][2];
  rotator[1][0] = rotator[2][0] = rotator[0][1] = rotator[0][2] = 0;
  rotator[0][0] = 1;

  int top = numSides;

  for(int i=0; i< top; i++)
    for(int j=0; j<3; j++)
      vertex[i][j] = rotator[j]*temp[i];
}

void knots::rotaterawnegyz(const double & cosangle)
{
  shortknot temp(*this);
  point rotator[3];
  rotator[1][1] = cosangle;
  rotator[2][2] = rotator[1][1];
  rotator[1][2] = -sqrt(1-cosangle*cosangle);
  rotator[2][1] = -rotator[1][2];
  rotator[1][0] = rotator[2][0] = rotator[0][1] = rotator[0][2] = 0;
  rotator[0][0] = 1;

  int top = numSides;

  for(int i=0; i< top; i++)
    for(int j=0; j<3; j++)
      vertex[i][j] = rotator[j]*temp[i];
}

void knots::rotatezx(const double & phi)
{
  shortknot temp(*this);
  point rotator[3];
  rotator[0][0] = cos(phi);
  rotator[2][2] = rotator[0][0];
  rotator[0][2] = sin(phi);
  rotator[2][0] = -rotator[0][2];
  rotator[1][0] = rotator[1][2] = rotator[0][1] = rotator[2][1] = 0;
  rotator[1][1] = 1;

  int top = numSides;

  for(int i=0; i< top; i++)
    for(int j=0; j<3; j++)
      vertex[i][j] = rotator[j]*temp[i];
}


void knots::rotateyz(const double & rho, const int & i, const int & j)
{
  /* precondition: i<j */

  shortknot temp(*this);
  point rotator[3];
  rotator[1][1] = cos(rho);
  rotator[2][2] = rotator[1][1];
  rotator[2][1] = sin(rho);
  rotator[1][2] = -rotator[2][1];
  rotator[0][1] = rotator[0][2] = rotator[1][0] = rotator[2][0] = 0;
  rotator[0][0] = 1;

  for(int k=i+1; k < j; k++)
    for(int n=0; n<3; n++)
      vertex[k][n] = rotator[n]*temp[k];
}

void knots::Compute()
{
  polyLength=0;
  epsilon = 64000;
  int last = numSides - 1;

  /* last is here to speed computations by avoiding modulo issues */

  for(int i=0; i<last; i++)
    {
      side[i] = vertex[i+1] - vertex[i];
      lenvtr[i] = side[i].norm();
      if(lenvtr[i]<epsilon)
	epsilon = lenvtr[i];
      polyLength += lenvtr[i];
    }
  
  side[last] = vertex[0] - vertex[last];
  lenvtr[last] = side[last].norm();
  if(lenvtr[last]<epsilon)
    epsilon = lenvtr[last];
  polyLength += lenvtr[last];

  ThicknessData();
}


void knots::ThicknessData()
{
  const double BIGNUMBER = 65000;
  int k,i,j;
  double maxCurvature=0,sumangle, totalcurvature, dummy;
  double a,b,d;
  R[3] = R[2] = R[1] = BIGNUMBER;
    
  /* this part avoids the modulo issues */

  double value = (side[numSides-1]*side[0]) / (lenvtr[numSides-1]*lenvtr[0]);
  if (value >= 1)
    angle[0] = 0;
  else if (value <= -1)
    angle[0] = M_PI;
  else
    {
      angle[0]=acos(value);
      totalcurvature = angle[0];
      double num = (2*tan(angle[0]*.5));
      /* check to see if curvature is bigger than the maximum */
      dummy = num/lenvtr[0];
      if(dummy>maxCurvature)
	maxCurvature=dummy;
      dummy = (num/lenvtr[numSides-1]);
      if(dummy>maxCurvature)
	maxCurvature=dummy;
    }
	
  for (k=1; k<numSides; k++)
    {
      /* angle[k] is the exterior angle at vertex[k] which is the 
	 angle between side[k-1] and side[k] */
      /* if(side[k-1] != side[k]) */
      double value = (side[k-1]*side[k]) / (lenvtr[k-1]*lenvtr[k]);
      if (value >= 1)
	angle[k] = 0;
      else if (value <= -1)
	angle[k] = M_PI;
      else
	{
	  angle[k]=acos(value);
	  totalcurvature += angle[k];
	  double num = (2*tan(angle[k]/2));
	  dummy = num/lenvtr[k];
	  /* check to see if curvature is bigger than the maximum */
	  if(dummy > maxCurvature)
	    maxCurvature=(num/lenvtr[k]);
	  dummy = (num/lenvtr[k-1]);
	  if(dummy > maxCurvature)
	    maxCurvature=dummy;
	}
	
      /* compute the Randell and Frohmann version of curvature
         a = lenvtr[k];
         b = lenvtr[(k+1) % vnum];
         theta = angle[(k+1) % vnum];
         alpha = fabs(atan( sin(theta) / ( a / b + cos(theta))));
         x = b / (2 * sin(alpha));

         if (x < altminRad)
           altminRad = x; */
    }

  R[0] = (1/maxCurvature);
  if(minRad_factor != 0)
    injRadius =  R[0] * minRad_factor;
  else 
    injRadius = 65000;

  int top = numSides - 1;

  for (i=0; i<top; i++)
    {
      sumangle = angle[i];
      k = i;
      j = numSides - 1;    /* this is so the computation below is quick */

      do
	{
	  k++;
	  sumangle += angle[k];
	}while((sumangle<M_PI) && (k<j));

      sumangle = totalcurvature + angle[i] + angle[k] - sumangle;
      /* cout << "i " << i << " starts at j " << k << endl;
      cout << "now backwards tc is " << sumangle; */

      for (j=k; (j<numSides) && (sumangle >= M_PI); j++)
	     
	{
	  sumangle -= angle[j];

	  if (( (j-i)>1 ) && !( (i==0) && (j==top)))
	  {
	    /* new IntToInt starts here */
	    if (find_rate(i,j,a,b,d) && (d<R[1]))
	      R[1] = d;
	    

	    /* start VertToInt here, i vertex, j side */
	    point e=(vertex[i]-vertex[j]);
	    a=(e*side[j])/(lenvtr[j]*lenvtr[j]);
	    if (0<=a && a<=1)
	      {
		if(IsDC(i,j,a))
		  {
		    d=(vertex[i]+(a-1)*vertex[j]+(-a)*vertex[((j+1)%numSides)]).norm();
		    if(d<R[2])
		      R[2]=d;
		  }
	      }
	    /* j vertex, i side */
	    e=(vertex[j]-vertex[i]);
	    a=(e*side[i])/(side[i]*side[i]);
	    if (0<=a && a<=1)
	      {
		if(IsDC(j,i,a))
		  {
		    d=(vertex[j]+(a-1)*vertex[i]+(-a)*vertex[((i+1)%numSides)]).norm();
		    if(d<R[2])
		      R[2]=d;
		  }
	      }
	    
	    /* start VertToVert here */
	    if(IsDC(i,j,0) && IsDC(j,i,0))
	      {
		d=(vertex[i]-vertex[j]).norm();
		if (d<R[3])
		  R[3]=d;
	      }
	    /* cout << "i " << i << " j " << j << " yes!\n"; */
	  }
	  /* else
	    cout << "i " << i << " j " << j << " no!\n"; */
	}
    }

  for(i=1;i<4;i++)
    {
      R[i]*=.5;
      if(R[i]<injRadius)
	injRadius = R[i];
    }

  if (injRadius < epsilon)
    epsilon = injRadius;
}


int knots::find_rate(const int & i, const int & j, 
		     double & a, double & b, double & d) const
{ 
  double vv;
  point v;

  v = side[i] & side[j];
  if ((vv = v * v)!= 0) {
    vv = 1/vv;
    a = (((vertex[j] - vertex[i]) & side[j]) * v) * vv;
    b = (((vertex[j] - vertex[i]) & side[i]) * v) * vv;
   }
  else 
    {
      double dummy = 1 / (side[i] * side[i]);
      double c = (vertex[j] - vertex[i]) * side[i] * dummy;
      double d = ((vertex[((j+1)%numSides)] - vertex[i]) * side[i]) * dummy;
      if ( (c<0 && d<0) || (c>1 && d>1))
	a=-1, b=-1;
      else if (0<=c && 1>=c)
        a=c, b=0;
      else if (0<=d && 1>=d)
	a=d, b=1;
      else 
	{
	  a = 0;
	  b = (vertex[i] - vertex[j]) * side[j] / (side[j]*side[j]);
	}
    }

  if (0 <= a && a<=1 && 0<=b && b<=1)
    {
      d = ((vertex[i] + a * side[i]) - (vertex[j] + b * side[j])).norm();
      return 1;
    }
  else 
    return 0;
  
}

int knots::IsDC(const int & i, const int & j, const double & a) const
{
  point v=(vertex[j]+a*side[j]-vertex[i]);

  int s;

  if(i>0)
    s=( (v * side[i-1]) >= 0);
  else
    s=( (v * side[numSides-1]) >= 0);
  
  int t=(v*side[i] <= 0);
  if (s == t)
    return 1;
  else
    return 0;
}


void knots::Display()
{ 
  cout.setf(ios::fixed);
  cout.precision(10);  // Changed from 6 in original -- JC.

  cout << "\nInterior to interior: " << R[1] << endl;  
  cout << "Vertex to interior: " << R[2] << endl;
  cout << "Vertex to vertex: " << R[3] << endl;
  cout << "Radius of curvature: " << R[0] << endl;
  cout << "Radius of curvature * factor: " << R[0]*minRad_factor << endl;
  cout << "Polygon Length: " << polyLength << endl;
  cout << "Injectivity Radius: " << injRadius << endl;
  cout << "Thickness: " << injRadius/polyLength << endl;
  cout << "Rope Length: " << polyLength/injRadius << endl << endl; 

  cout.precision(6);
}

double knots::injrad() const
{
  return injRadius;
}

double knots::RopeLength() const
{
  return polyLength/injRadius;
}

double knots::Thickness() const
{
  return injRadius/polyLength;
}

double knots::Epsilon() const
{
  return epsilon;
}

double knots::MinRad() const
{
  return R[0];
}

double knots::Length() const
{
  return polyLength;
}

double knots::minRadFactor() const
{
  return minRad_factor;
}

double knots::InteriorInterior() const
{
  return R[1];
}

double knots::VertexInterior() const
{
  return R[2];
}

double knots::VertexVertex() const
{
  return R[3];
}

double knots::ChangeMinRadFactor(double newfactor)
{
  if(newfactor > 1)
    cerr << "The minRad factor cannot be bigger than 1.\n"
      << "The minRad factor remains at " << minRad_factor << endl;
  else if(newfactor < 0)
    cerr << "The minRad factor cannot be smaller than 0.\n"
      << "The minRad factor remains at " << minRad_factor << endl;
  else
    minRad_factor = newfactor;

  Compute();

  return minRad_factor;
}


void knots::BeShaken(const vector & shaker)
{
  int i,j;
  
  for(i=0; i<numSides; i++)
    for(j=0; j<3; j++)
      vertex[i][j] = vertex[i][j] + shaker[i][j];
}

void knots::SideData() const
{
  cout << "\nKnot has " << numSides << " sides.\n";

  for(int i=0; i<numSides; i++)
    {
      cout << "Side " << i << "  " << side[i] << "  length " << lenvtr[i]
	<< endl;
    }
  cout << endl;
}

void knots::AngleData() const
{
  cout << endl;
  for(int k=0; k<numSides; k++)
    cout << "Angle " << k << " = " << angle[k] << endl;
  cout << endl;
}

void knots::CurvatureData() const
{
  double totalCurvature = 0;
  double leftTotal = 0, rightTotal = 0;
  cout << endl;
  
  cout << "Data below includes a curvature factor of " 
       << minRad_factor << endl;
  for(int k=0; k<numSides; k++)
    {
      totalCurvature += angle[k];
      leftTotal += lenvtr[((k-1+numSides)%numSides)]/(2*tan(angle[k]*.5));
      rightTotal += lenvtr[k]/(2*tan(angle[k]*.5));
      cout << "angle[" << k << "] = " << angle[k];
      cout << "\tr-[" << k << "] = " 
	   << minRad_factor * lenvtr[((k-1+numSides)%numSides)]/(2*tan(angle[k]*.5));
      cout << "\tr+[" << k << "] = " 
	   << minRad_factor * lenvtr[k]/(2*tan(angle[k]*.5)) << endl;
    }
  cout << "Total curvature of knot is " << totalCurvature << endl << endl;
  cout << "Average - curvature is " 
       << minRad_factor * leftTotal/numSides << endl;
  cout << "Average + curvature is " 
       << minRad_factor * rightTotal/numSides << endl;
}

void knots::CurvatureSpreadsheet() const
{
  cout << "Dump to what file: ";
  char file[100];
  cin >> file;

  ofstream fout(file);

  fout << "BackCurvature,NextCurvature\n";

  for(int k=0; k<numSides; k++)
    {
      fout << (2*tan(angle[k]*.5))/lenvtr[((k-1+numSides)%numSides)]
	   << ",";
      fout << (2*tan(angle[k]*.5))/lenvtr[k]
	   << endl;
    }

  fout.close();
}

void knots::CriticalRelations()
{
  cout.setf(ios::fixed);
  cout.precision(15);
  polyLength=0;
  epsilon = 64000;
  int last = numSides - 1;
  int k,i,j;

  /* last is here to speed computations by avoiding modulo issues */

  for(i=0; i<last; i++)
    {
      side[i] = vertex[i+1] - vertex[i];
      lenvtr[i] = side[i].norm();
      if(lenvtr[i]<epsilon)
	epsilon = lenvtr[i];
      polyLength += lenvtr[i];
    }
  
  side[last] = vertex[0] - vertex[last];
  lenvtr[last] = side[last].norm();
  if(lenvtr[last]<epsilon)
    epsilon = lenvtr[last];
  polyLength += lenvtr[last];

  const double BIGNUMBER = 65000;
  double maxCurvature=0,sumangle, totalcurvature, dummy;
  double a,b,d;
  R[3] = R[2] = R[1] = BIGNUMBER;
    
  /* this part avoids the modulo issues */

  double value = (side[numSides-1]*side[0]) / (lenvtr[numSides-1]*lenvtr[0]);
  if (value >= 1)
    angle[0] = 0;
  else if (value <= -1)
    angle[0] = M_PI;
  else
    {
      angle[0]=acos(value);
      totalcurvature = angle[0];
      double num = (2*tan(angle[0]*.5));
      /* check to see if curvature is bigger than the maximum */
      dummy = num/lenvtr[0];
      if(dummy>maxCurvature)
	maxCurvature=dummy;
      dummy = (num/lenvtr[numSides-1]);
      if(dummy>maxCurvature)
	maxCurvature=dummy;
    }
	
  for (k=1; k<numSides; k++)
    {
      /* angle[k] is the exterior angle at vertex[k] which is the 
	 angle between side[k-1] and side[k] */
      /* if(side[k-1] != side[k]) */
      double value = (side[k-1]*side[k]) / (lenvtr[k-1]*lenvtr[k]);
      if (value >= 1)
	angle[k] = 0;
      else if (value <= -1)
	angle[k] = M_PI;
      else
	{
	  angle[k]=acos(value);
	  totalcurvature += angle[k];
	  double num = (2*tan(angle[k]/2));
	  dummy = num/lenvtr[k];
	  /* check to see if curvature is bigger than the maximum */
	  if(dummy > maxCurvature)
	    maxCurvature=(num/lenvtr[k]);
	  dummy = (num/lenvtr[k-1]);
	  if(dummy > maxCurvature)
	    maxCurvature=dummy;
	}
	
      /* compute the Randell and Frohmann version of curvature
         a = lenvtr[k];
         b = lenvtr[(k+1) % vnum];
         theta = angle[(k+1) % vnum];
         alpha = fabs(atan( sin(theta) / ( a / b + cos(theta))));
         x = b / (2 * sin(alpha));

         if (x < altminRad)
           altminRad = x; */
    }

  int top = numSides - 1;

  vectors dists(numSides,65000);
  vectors vdists(numSides,65000);
  vectors vidists(numSides,65000);
  vectors vvdists(numSides,65000);
  vectors bestdists(numSides,65000);
  vectors pair(numSides,65000);
  vectors vpair(numSides,65000);
  vectors vipair(numSides,65000);
  vectors vvpair(numSides,65000);
  vectors type(numSides,65000);

  for (i=0; i<numSides; i++)
    {
      sumangle = angle[i];
      k = i;

      do
	{
	  k++;
	  sumangle += angle[k%numSides];
	}while(sumangle<M_PI);

      sumangle = totalcurvature + angle[i] + angle[k] - sumangle;

      for (j=k; (j<i+numSides) && (sumangle >= M_PI); j++)
	{
	  int jnew = j%numSides;
	  sumangle -= angle[jnew];

	  if ( ((abs(jnew-i)) > 1) && (abs(j-numSides-i) > 1) )
	  {
	    /* new IntToInt starts here */
	    if (find_rate(i,jnew,a,b,d))
	      {
		if(d<dists[i]){
		  dists[i]=d;
		  pair[i]=jnew;
		  if(d<bestdists[i]){
		    type[i]=0;
		    bestdists[i]=d;
		  }
		}
	      }
	    
	    /* start VertToInt here, i vertex, j side */
	    point e=(vertex[i]-vertex[jnew]);
	    a=(e*side[jnew])/(lenvtr[jnew]*lenvtr[jnew]);
	    if (0<=a && a<=1)
	      {
		if(IsDC(i,jnew,a))
		  {
		    d=(vertex[i]+(a-1)*vertex[jnew]+(-a)*vertex[((jnew+1)%numSides)]).norm();
		    if(d<R[2])
		      R[2]=d;
		    if(d<vdists[i]){
		      vdists[i]=d;
		      vpair[i]=jnew;
		      if(d<bestdists[i]){
			type[i]=1;
			bestdists[i]=d;
		      }
		    }
		  }
	      }
	    /* j vertex, i side */
	    e=(vertex[jnew]-vertex[i]);
	    a=(e*side[i])/(side[i]*side[i]);
	    if (0<=a && a<=1)
	      {
		if(IsDC(jnew,i,a))
		  {
		    d=(vertex[jnew]+(a-1)*vertex[i]+(-a)*vertex[((i+1)%numSides)]).norm();
		    if(d<vidists[i]){
		      vidists[i]=d;
		      vipair[i]=jnew;
		      if(d<bestdists[i]){
			type[i]=2;
			bestdists[i]=d;
		      }
		    }
		  }
	      }


	    /* start VertToVert here */
	    if(IsDC(i,jnew,0) && IsDC(jnew,i,0))
	      {
		d=(vertex[i]-vertex[jnew]).norm();
		if(d<vvdists[i]){
		  vvdists[i]=d;
		  vvpair[i]=jnew;
		  if(d<bestdists[i]){
		    type[i]=3;
		    bestdists[i]=d;
		  }
		}
	      }
	  }
	}
    }


  for(i=0; i<numSides; i++){
    if(dists[i]==65000)
      cout << "No doubly critical pairs with edge " << i << endl;
    else{
      if(type[i]==0)
	cout << "Int to int ";
      else if (type[i]==1)
	cout << "Vert to int ";
      else if (type[i]==2)
	cout << "Int to vert ";
      else if (type[i]==3)
	cout << "Vert to vert ";
      else
	cout << "Problem!\n";
      
      cout << "(" << i << "," << int(pair[i]) << ") = " << dists[i] << "\n";
    }
  }

  ThicknessData();
}




